/***************************************************************************
 *            mw.c
 *
 *  Sat Sep  4 21:28:07 2004
 *  Copyright  2004  Marcin K. Krzywonos
 *  mkk@gnutu.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <pwd.h>

#include "var.h"
#include "mw.h"
#include "gtudb.h"


/*
 * Funkcja wykonuje operacje przy pokazaniu okna main_window
 */
int mw_show (GtkWidget *widget)
{
	load_entry = TRUE;
	
	mw_tests_clist_rowselected  = -1;
	mw_quizes_clist_rowselected = -1;
	
	// Widget okna glownego i ustawiamy w nim kolory
	mw_widget = widget;
	
	// Wczytujemy bazy danych
	if (gtudb_read () == 1)
		idbw_show ();
	
	if (gtudb_settings_read () == 1)
		sw_default_settings ();
	
	gtudb_plan_read ();
	gtudb_fd_read ();
	
	// Wczytujemy wpis
	mw_get_entry_date ();
	mw_edit_entry (entry_date, FALSE);
	
	
	// Powitanie
	printf (_("--------------------\nGNU Terminarz Ucznia\n   www.gnutu.org\n--------------------\n\n"));
}


/*
 * Funkcja wczytuje wpis
 * Parametry:
 *            - date - data wczytywanego wpisu
 *            - save - okresla czy poprzedni wpis powinien byc zapisany
 */
int mw_edit_entry (type_date date, gboolean save)
{
	GSList       *tmp_db, *tmp_db_2; // Listy
	GTUDBEntry   *tmp_entry;         // Zmienne przechowujace wpis
	GTUDBSubject *tmp_sub;
	int           sub_cnt   = 0;     // Licznik
	
	if (save == TRUE) {
		// Odczytujemy dane biezacego wpisu do zmiennej r_entry
		GTUDBEntry *r_entry   = g_new (GTUDBEntry, 1);
		
		r_entry->subjects = mw_marks_clist_get_marks   ();
		r_entry->notes    = mw_notes_textview_get_text ();
		r_entry->date     = entry_date;
		
		// Usuwamy element listy o numerze aktualnie wczytanego wpisu
		GSList *temp_ery = g_slist_nth (gtudb_list, gtudb_search_entry (entry_date));
		gtudb_list = g_slist_remove_link (gtudb_list, temp_ery);
		
		// Dodajemy nowy element
		gtudb_list = g_slist_append (gtudb_list, r_entry);
	}
	
	// Numer wczytywanego wpisu (o ile istnieje)
	int entry_cnt = gtudb_search_entry (date);
	
	if (entry_cnt == -1) { // Jesli nie istnieje to tworzymy nowy, pusty wpis
		mw_clear_all ();
		
		// automatyczne wstawianie przedmiotow
		if (strcmp (gtudb_settings_get_value ("insert_subjects"), "TRUE") == 0) {
			// sprawdzamy czy wybrana data nie "znajduje" sie na wakacjach
			if ((g_date_compare (text_to_gdate (gtudb_settings_get_value ("school_year_end")),
								type_date_to_gdate (date)) > 0) &&
			   (g_date_compare (text_to_gdate (gtudb_settings_get_value ("vacation_end")),
								type_date_to_gdate (date)) > 0))
			{
				if (gtudb_fd_search_day (date) == -1) // sprawdzamy czy data jest wolnym dniem
					mw_marks_clist_load_subjects (date_get_weekday (date));
			}
		}
	} else { // W przeciwnym wypadku wczytujemy stary wpis
		// Pobieramy wpis z listy
		tmp_db    = g_slist_nth (gtudb_list, entry_cnt);
		tmp_entry = tmp_db->data;
		
		// Czyscimy zawartosc wszystkich pol wpisu
		mw_clear_all ();
		
		// Wypelniamy pola wpisu
		mw_notes_textview_set_text (tmp_entry->notes);
		
		while (sub_cnt < g_slist_length (tmp_entry->subjects)) {
			tmp_db_2 = g_slist_nth (tmp_entry->subjects, sub_cnt);
			tmp_sub  = tmp_db_2->data;
			
			mw_marks_clist_add (tmp_sub->time, tmp_sub->name, tmp_sub->marks);
			
			if (tmp_sub->flag == 1)
				mw_quizes_clist_add (tmp_sub->name);
			
			if (tmp_sub->flag == 2)
				mw_tests_clist_add (tmp_sub->name);
			
			sub_cnt++;
		}
	}
	
	// Ustawiamy biezaca date na date wczytanego lub utworzonego nowego wpisu
	entry_date = date;
	
	// pozostale informacje
	mw_date_label_set_date (entry_date);
	mw_info_label_set_info ();
}


/*
 * Funkcja zapisuje w zmiennej 'entry_date' date biezacego wpisu
 */
int mw_get_entry_date ()
{
	GtkWidget *calendar = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_calendar");
	gtk_calendar_get_date (GTK_CALENDAR (calendar), &entry_date.year, &entry_date.month, &entry_date.day);
	
	entry_date.month++;
}


/*
 * Funkcja ustawia date w etykiecie mw_date_label_set_date
 */
int mw_date_label_set_date (type_date date)
{
	GtkWidget *mw_date_label = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_date_label");
	
	gchar tmp[100];
	sprintf (tmp, "<b><span size='x-large'>%d-%d-%d</span></b>", date.day, date.month, date.year);
	
	gtk_label_set_label (GTK_LABEL (mw_date_label), tmp);
	
	return 0;
}


/*
 * Funkcja wyswietla informacje o wpisie w etykiecie mw_info_label
 */
int mw_info_label_set_info ()
{
	GtkWidget *mw_info_day_label        = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_info_day_label");
	GtkWidget *mw_tests_label           = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tests_label");
	GtkWidget *mw_notes_vbox            = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_notes_vbox");
	GtkWidget *mw_subjects_vbox         = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_subjects_vbox");
	GtkWidget *mw_diary_vbox            = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_diary_vbox");
	GtkWidget *mw_label_info_days_vbox  = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_label_info_days_vbox");
	GtkWidget *mw_label_info_tests_vbox = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_label_info_tests_vbox");
	
	
	if (strcmp (gtudb_settings_get_value ("show_info_about_day"), "TRUE") == 0) {
		gtk_widget_show (GTK_WIDGET (mw_info_day_label));
		mw_info_day ();
	} else
		gtk_widget_hide (GTK_WIDGET (mw_info_day_label));
	
	if (strcmp (gtudb_settings_get_value ("show_new_tests"), "TRUE") == 0) {
		gtk_widget_show (GTK_WIDGET (mw_label_info_tests_vbox));
		mw_info_search_test ();
	} else
		gtk_widget_hide (GTK_WIDGET (mw_label_info_tests_vbox));
	
	if (strcmp (gtudb_settings_get_value ("show_days_counter"), "TRUE") == 0) {
		gtk_widget_show (GTK_WIDGET (mw_label_info_days_vbox));
		mw_info_days_cnt ();
	} else
		gtk_widget_hide (GTK_WIDGET (mw_label_info_days_vbox));
	
	if (strcmp (gtudb_settings_get_value ("show_diary"), "TRUE") == 0) {
		gtk_widget_show (GTK_WIDGET (mw_diary_vbox));
		mw_info_from_diary ();
	} else
		gtk_widget_hide (GTK_WIDGET (mw_diary_vbox));
	
	if (strcmp (gtudb_settings_get_value ("show_tests_today"), "TRUE") == 0) {
		gtk_widget_show (GTK_WIDGET (mw_tests_label));
		mw_info_todays_tests ();
	} else
		gtk_widget_hide (GTK_WIDGET (mw_tests_label));
	
	if (strcmp (gtudb_settings_get_value ("show_subjects"), "TRUE") == 0) {
		gtk_widget_show (GTK_WIDGET (mw_subjects_vbox));
		mw_info_subjects ();
	} else
		gtk_widget_hide (GTK_WIDGET (mw_subjects_vbox));
		
	if (strcmp (gtudb_settings_get_value ("show_notes"), "TRUE") == 0) {
		gtk_widget_show (GTK_WIDGET (mw_notes_vbox));
		mw_info_notes ();
	} else
		gtk_widget_hide (GTK_WIDGET (mw_notes_vbox));
	
	return 0;
}


/*
 * Funkcja wypisyje informacje o sprawdzianach
 */
int mw_info_search_test ()
{
	gchar     rtn[2048];
	GtkWidget *mw_label_info_tests = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_label_info_tests");
	sprintf (rtn, "");
	
	GDate *begin_date = type_date_to_gdate (entry_date);
	g_date_add_days (begin_date, 1);
	
	GDate *end_date = type_date_to_gdate (entry_date);
	g_date_add_days (end_date, atoi (gtudb_settings_get_value ("show_new_tests_days")));
	
	if (g_slist_length (gtudb_search_test (gdate_to_type_date (begin_date), gdate_to_type_date (end_date))) > 0) {
		type_date date_tmp = gdate_to_type_date (text_to_gdate (g_strdup (g_slist_nth (gtudb_search_test (gdate_to_type_date (begin_date), gdate_to_type_date (end_date)), 0)->data)));
		sprintf (rtn, "%d-%d-%d\n", date_tmp.day, date_tmp.month, date_tmp.year);
		
		GSList *tmp2 = gtudb_entry_get_subjects_list (date_tmp, 2);
		int j    = 0;
		while (j < g_slist_length (tmp2)) {
			sprintf (rtn, "%s\t%s\n", rtn, g_strdup (g_slist_nth (tmp2, j)->data));
			j++;
		}
		
		tmp2 = gtudb_entry_get_subjects_list (date_tmp, 1);
		j    = 0;
		while (j < g_slist_length (tmp2)) {
			sprintf (rtn, "%s\t%s\n", rtn, g_strdup (g_slist_nth (tmp2, j)->data));
			j++;
		}
	}
	
	
	GtkWidget *mw_label_info_tests_vbox = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_label_info_tests_vbox");
	
	if (strlen (rtn) == 0) {
		gtk_widget_hide (GTK_WIDGET (mw_label_info_tests_vbox));
		return 1;
	} else {
		gtk_widget_show (GTK_WIDGET (mw_label_info_tests_vbox));
	}
	
	gtk_label_set_label (GTK_LABEL (mw_label_info_tests), rtn);
	
	return 0;
}


/*
 * Funkcja zapisuje do etykiety mw_label_info_days informacje o ilosci dni
 * pozostalych do konca roku szkolnego.
 */
int mw_info_days_cnt ()
{
	GtkWidget *mw_label_info_days   = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_label_info_days");
	GtkWidget *mw_label_info_days_y = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_label_info_days_y");
	GtkWidget *mw_label_info_days_v = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_label_info_days_v");
	gchar      rtn[10];
	
	if (date_diff (entry_date, gdate_to_type_date (text_to_gdate (gtudb_settings_get_value ("school_year_end")))) <= 0) {
		if (date_diff (entry_date, gdate_to_type_date (text_to_gdate (gtudb_settings_get_value ("vacation_end")))) > 0) {
			sprintf (rtn, _("%d"), date_diff (entry_date, gdate_to_type_date (text_to_gdate (gtudb_settings_get_value ("vacation_end")))));
			
			gtk_widget_show (GTK_WIDGET (mw_label_info_days_v));
			gtk_widget_hide (GTK_WIDGET (mw_label_info_days_y));
		} else {
			gtk_widget_hide (GTK_WIDGET (mw_label_info_days_v));
			gtk_widget_hide (GTK_WIDGET (mw_label_info_days_y));
			sprintf (rtn, "");
		}
	} else {
		sprintf (rtn, _("%d"), date_diff (entry_date, gdate_to_type_date (text_to_gdate (gtudb_settings_get_value ("school_year_end")))));
		
		gtk_widget_show (GTK_WIDGET (mw_label_info_days_y));
		gtk_widget_hide (GTK_WIDGET (mw_label_info_days_v));
	}
	
	gtk_label_set_text (GTK_LABEL (mw_label_info_days), rtn);
	
	return 0;
}


/*
 * Funkcja zapisuje do etykiety mw_label_info_days informacje o ilosci dni
 * pozostalych do konca roku szkolnego.
 */
int mw_info_from_diary ()
{
	GtkWidget *mw_from_diary_label = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_from_diary_label");
	
	gchar fname [512];
	sprintf (fname, "%s/.gnutu/diary/%d-%d-%d.gtu", g_get_home_dir(), entry_date.day, entry_date.month, entry_date.year);
	
	FILE *file = fopen (fname, "r");
	gchar tmp [100];
	gchar chr;
	int   i    = 0;
	
	strcpy (tmp, "");
	
	if (file != NULL) {
		chr = fgetc (file);
		
		while (!feof (file) && (i <= 96)) {
			if (chr != '\n')
				sprintf (tmp, "%s%c", tmp, chr);
			
			chr = fgetc (file);
			i++;
			
		}
		
		fclose (file);
	}
	
	if (strlen (tmp) > 0) {
		if (strlen (tmp) >= 96)
			strcat (tmp, "...");
		
		gtk_label_set_label (GTK_LABEL (mw_from_diary_label), tmp);
	} else {
		sprintf (tmp, "<i>%s</i>", _("None"));
		gtk_label_set_label (GTK_LABEL (mw_from_diary_label), tmp);
	}
}


/*
 * Funkcja zapisuje do etykiety mw_label_todays_test informacje sprawdzianach
 * i kartkowkach w aktualnym dniu
 */
int mw_info_todays_tests ()
{
	GtkWidget *mw_tests_label  = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tests_label");
	GtkWidget *mw_tests_clist  = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tests_clist");
	GtkWidget *mw_quizes_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_quizes_clist");
	
	gchar     *tmp;
	GString   *lbl = g_string_new ("");
	
	int i = 0;
	while (gtk_clist_get_text (GTK_CLIST (mw_tests_clist), i, 0, &tmp) == 1) {
		if (i == 0)
			g_string_append_printf (lbl, "<b>%s:</b>\n%s\n", _("Class-tests"), tmp);
		else
			g_string_append_printf (lbl, "%s\n", tmp);
		
		i++;
	}
	
	i = 0;
	while (gtk_clist_get_text (GTK_CLIST (mw_quizes_clist), i, 0, &tmp) == 1) {
		if (i == 0) {
			if (strlen (lbl ->str) > 0)
				lbl = g_string_append (lbl, "\n");
			
			g_string_append_printf (lbl, "<b>%s:</b>\n%s\n", _("Quizes"), tmp);
		} else
			g_string_append_printf (lbl, "%s\n", tmp);
		
		i++;
	}
	
	
	if (strlen (lbl ->str) > 0) {
		gtk_widget_show (GTK_WIDGET (mw_tests_label));
		gtk_label_set_label (GTK_LABEL (mw_tests_label), lbl->str);
	} else
		gtk_widget_hide (GTK_WIDGET (mw_tests_label));
	
	return 0;
}


/*
 * Funkcja wpisuje do etykiety mw_info_subjects przedmioty z danego dnia
 */
int mw_info_subjects ()
{
	GtkWidget *mw_subjects_label = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_subjects_label");
	GtkWidget *mw_subjects_vbox  = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_subjects_vbox");
	GtkWidget *mw_marks_clist    = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_marks_clist");
	
	gchar     *tmp;
	GString   *lbl = g_string_new ("");
	
	int i = 0;
	while (gtk_clist_get_text (GTK_CLIST (mw_marks_clist), i, 1, &tmp) == 1) {
		g_string_append_printf (lbl, "%s\n", tmp);
		i++;
	}
	
	if (strlen (lbl ->str) > 0) {
		gtk_widget_show (GTK_WIDGET (mw_subjects_vbox));
		gtk_label_set_label (GTK_LABEL (mw_subjects_label), lbl->str);
	} else
		gtk_widget_hide (GTK_WIDGET (mw_subjects_vbox));
}


/*
 * Funckja wpisuje do etykiety mw_info_notes skrocona wersje tekstu z notesu
 */
int mw_info_notes ()
{
	GtkWidget *mw_notes_label = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_notes_label");
	GtkWidget *mw_notes_vbox  = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_notes_vbox");
	gchar     *tmp            = mw_notes_textview_get_text ();
	
	
	if (strlen (tmp) > 0) {
		GString *lbl = g_string_new ("");
		
		int i = 0;
		while ((i < strlen (tmp)) && (i < 100)) {
			if (tmp [i] != '\n')
				lbl = g_string_append_c (lbl, tmp [i]);
			
			i++;
		}
		
		if (strlen (lbl->str) >= 96)
			lbl = g_string_append (lbl, "...");
		
		gtk_label_set_text (GTK_LABEL (mw_notes_label), lbl->str);
		gtk_widget_show (GTK_WIDGET (mw_notes_vbox));
	} else {
		gtk_widget_hide (GTK_WIDGET (mw_notes_vbox));
	}
	
	return 0;
}


/*
 * Funkcja wpisuje do etykiety mw_info_label informacje o dniu
 */
int mw_info_day ()
{
	GtkWidget *mw_info_day_label = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_info_day_label");
	gchar      rtn[2048];
	
	gtk_widget_hide (GTK_WIDGET (mw_info_day_label));
	
	if (gtudb_fd_search_day (entry_date) == -1) {
		gtk_label_set_label (GTK_LABEL (mw_info_day_label), "");
		return 1;
	}
	
	GTUDBFreeDays *tmp_entry = tmp_entry = g_slist_nth (gtudb_fd_list, gtudb_fd_search_day (entry_date))->data;
	
	if (strlen (tmp_entry->desc) < 1) {
		gtk_label_set_label (GTK_LABEL (mw_info_day_label), "");
		return 0;
	}
	
	strcpy (rtn, "<i>");
	strcat (rtn, (gchar *) g_strdup (tmp_entry->desc));
	strcat (rtn, "</i>");
	
	gtk_label_set_label (GTK_LABEL (mw_info_day_label), rtn);
	gtk_widget_show (GTK_WIDGET (mw_info_day_label));
	
	return 0;
}


/*
 * Funkcja sprawia, ze przycisk "btn_name" (do wybierania zakladki)
 * jest wlaczony, a pozostale przyciski sa wylaczone.
 */
int mw_tab_set_active_btn (int id)
{
	GtkWidget *mw_tab_info_btn      = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tab_info_btn");
	GtkWidget *mw_tab_at_school_btn = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tab_at_school_btn");
	GtkWidget *mw_tab_stat_btn      = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tab_stat_btn");
	
	
	switch (id) {
		case 1: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw_tab_at_school_btn), FALSE);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw_tab_stat_btn), FALSE);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw_tab_info_btn), TRUE);
				break;
		case 2: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw_tab_info_btn), FALSE);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw_tab_stat_btn), FALSE);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw_tab_at_school_btn), TRUE);
				break;
		case 4: gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw_tab_info_btn), FALSE);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw_tab_at_school_btn), FALSE);
				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mw_tab_stat_btn), TRUE);
				break;
	}
	
	return 0;
}


/* Funkcja dodaje nowa pozycje do listy przedmiotow i ocen
 * Parametry:
 *           - time    - godzina rozpoczecia lekcji
 *           - subject - nazwa przedmiotu
 *           - marks   - oceny
 */
int mw_marks_clist_add (gchar *time, gchar *subject, gchar *marks)
{
	if (strlen (subject) == 0)
		return 1;
	
	gchar *tmp_time;
	if (time [1] == ':') {
		sprintf (tmp_time, "0%s", time);
		time = tmp_time;
	}
	
	GtkWidget *mw_marks_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_marks_clist");
	
	gchar *mw_tmp [] = {time, subject, marks};
	gtk_clist_insert (GTK_CLIST (mw_marks_clist), 0, mw_tmp);
	gtk_clist_sort (GTK_CLIST (mw_marks_clist));
	
	int      i = 0;
	GdkColor color  = {0, 0xEEEE, 0xEEEE, 0xEEEE};
	GdkColor color2 = {0, 0xFFFF, 0xFFFF, 0xFFFF};
	gdk_color_alloc (gdk_colormap_get_system (), &color);
	gdk_color_alloc (gdk_colormap_get_system (), &color2);
	
	while (gtk_clist_get_text (GTK_CLIST (mw_marks_clist), i, 1, NULL)) {
		if (i % 2 == 1)
			gtk_clist_set_background (GTK_CLIST (mw_marks_clist), i, &color);
		else
			gtk_clist_set_background (GTK_CLIST (mw_marks_clist), i, &color2);
		
		i++;
	}
	
	return 0;
}



/*
 * Funkcja usuwa pozycje o numerze id z listy przedmiotow i ocen
 */
int mw_marks_clist_remove (int id)
{
	GtkWidget *mw_marks_clist  = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_marks_clist");
	
	gchar *tmp;
	gtk_clist_get_text (GTK_CLIST (mw_marks_clist), id, 1, &tmp);
	
	mw_tests_clist_remove_test (tmp);
	mw_tests_clist_remove_quiz (tmp);
	gtk_clist_remove (GTK_CLIST (mw_marks_clist), id);
}


/*
 * Funkcja czysci zawartosc listy przedmiotow i ocen*
 */
int mw_marks_clist_clear ()
{
	GtkWidget *mw_marks_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_marks_clist");
	gtk_clist_clear (GTK_CLIST (mw_marks_clist));
}


/*
 * Funkcja sprawdza czy na podanej liscie znajduje sie juz przedmiot 'subject'
 */
gboolean mw_check_subject (GSList *list, gchar *subject)
{
	GTUDBSubject *tmp_subject;
	int i = 0;
	
	while (i < g_slist_length (list)) {
		tmp_subject = g_new (GTUDBSubject, 1);
		tmp_subject = g_slist_nth (list, i)->data;
		
		if (strcmp (tmp_subject->name, subject) == 0)
			return TRUE;
		
		i++;
	}
	
	return FALSE;
}


/*
 * Funkcja pobiera i zwraca zawartosc pola przedmiotow i ocen
 */
GSList *mw_marks_clist_get_marks ()
{
	GtkWidget    *mw_marks_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_marks_clist");
	GSList       *list           = NULL;
	GTUDBSubject *subject        = g_new (GTUDBSubject, 1);
	
	int    i = 0;
	gchar *tmp;
	
	while (gtk_clist_get_text (GTK_CLIST (mw_marks_clist), i, 1, &tmp) == 1) {
		subject->name = g_strdup (tmp);
		
		gtk_clist_get_text (GTK_CLIST (mw_marks_clist), i, 0, &tmp);
		subject->time = g_strdup (tmp);
		
		gtk_clist_get_text (GTK_CLIST (mw_marks_clist), i, 2, &tmp);
		subject->marks = g_strdup (tmp);
		
		if (mw_tests_clist_check_test (subject->name) == 1) {
			if (mw_check_subject (list, g_strdup (subject->name)) == FALSE)
				subject->flag = 2;
		} else {
			if (mw_quizes_clist_check_quiz (subject->name) == 1) {
				if (mw_check_subject (list, g_strdup (subject->name)) == FALSE)
					subject->flag = 1;
			} else
				subject->flag = 0;
		}
		
		list = g_slist_append (list, subject);
		
		subject = g_new (GTUDBSubject, 1);
		i++;
	}
	
	return list;
}


/*
 * Funkcja wczytuje z planu lekcji do tabeli "mw_marks_clist" przedmioty
 * odpowiadajace dniu podanemu jako parametr day
 */
int mw_marks_clist_load_subjects (int day)
{
	if (day == 7)
		return 0;
	
	mw_marks_clist_clear ();
	
	int i = 0;
	while ((i < g_slist_length (gtudb_plan_list [0])) && (i < g_slist_length (gtudb_plan_list [day]))) {
		mw_marks_clist_add (g_strdup (g_slist_nth (gtudb_plan_list[0], i)->data),
							g_strdup (g_slist_nth (gtudb_plan_list[day], i)->data), "");
		
		i++;
	}
	
	return 0;
}


/*
 * Funkcja wczytuje z listy przedmiotow i ocen do pol wpisu przedmiotow i ocen
 * linie o numerze zapisanym w zmiennej "row"
 */
int mwes_marks_edit (int row)
{
	gchar *time, *subject, *marks;
	
	GtkWidget *mw_marks_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_marks_clist");
	gtk_clist_get_text (GTK_CLIST (mw_marks_clist), row, 0, &time);
	gtk_clist_get_text (GTK_CLIST (mw_marks_clist), row, 1, &subject);
	gtk_clist_get_text (GTK_CLIST (mw_marks_clist), row, 2, &marks);
	
	
	GtkWidget *mwes_subject_flag_1 = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_subject_flag_1");
	GtkWidget *mwes_subject_flag_2 = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_subject_flag_2");
	GtkWidget *mwes_subject_flag_3 = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_subject_flag_3");
	
	if (mw_tests_clist_check_test (subject) == 1) {
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mwes_subject_flag_1), TRUE);
	} else {
		if (mw_quizes_clist_check_quiz (subject) == 1)
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mwes_subject_flag_2), TRUE);
		else
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (mwes_subject_flag_3), TRUE);
	}
	
	GtkWidget *mwes_marks_time_entry = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_marks_time_entry");
	gtk_entry_set_text (GTK_ENTRY (mwes_marks_time_entry), time);
	
	GtkWidget *mwes_marks_subject_entry = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_marks_subject_entry");
	gtk_entry_set_text (GTK_ENTRY (mwes_marks_subject_entry), subject);
	
	GtkWidget *mwes_marks_marks_entry = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_marks_marks_entry");
	gtk_entry_set_text (GTK_ENTRY (mwes_marks_marks_entry), marks);
}


/*
 * Funkcja pobiera dane z pol wpisu przedmiotow i ocen i dodaje do listy
 */
int mwes_marks_add ()
{
	// Zmienne, itp...
	gchar *time, *subject, *marks;
	
	GtkWidget *mwes_subject_flag_1 = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_subject_flag_1");
	GtkWidget *mwes_subject_flag_2 = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_subject_flag_2");
	GtkWidget *mwes_marks_time_entry = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_marks_time_entry");
	GtkWidget *mwes_marks_subject_entry = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_marks_subject_entry");
	GtkWidget *mwes_marks_marks_entry = (gpointer) lookup_widget (GTK_WIDGET (mwes_widget), "mwes_marks_marks_entry");
	
	
	// Odczytujemy dane
	time    = (gchar *) gtk_entry_get_text (GTK_ENTRY (mwes_marks_time_entry));
	subject = (gchar *) gtk_entry_get_text (GTK_ENTRY (mwes_marks_subject_entry));
	marks   = (gchar *) gtk_entry_get_text (GTK_ENTRY (mwes_marks_marks_entry));
	
	
	// Uzupelniamy pola...
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (mwes_subject_flag_1)) == TRUE) {
		mw_tests_clist_add (subject);
	} else {
		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (mwes_subject_flag_2)) == TRUE)
			mw_quizes_clist_add (subject);
	}
	
	mw_marks_clist_add (time, subject, marks);
	
	
	// zapisujemy wpis
	mw_edit_entry (entry_date, TRUE);
	
	return 0;
}


/* Funkcja dodaje nowa pozycje do listy sprawdzianow
 * parametry:
 *            - gubject - nazwa_przedmiotu
 */
int mw_tests_clist_add (gchar *subject)
{
	if ((strlen (subject) == 0) || (mw_tests_clist_check_test (subject) > 0))
		return 1;
	
	GtkWidget *mw_tests_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tests_clist");
	gchar *mw_tmp [] = {subject};
	
	gtk_clist_insert (GTK_CLIST (mw_tests_clist), 0, mw_tmp);
	
	return 0;
}


/*
 * Czysci zawartosc listy klasowek
 */
int mw_tests_clist_clear ()
{
	GtkWidget *mw_tests_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tests_clist");
	gtk_clist_clear (GTK_CLIST (mw_tests_clist));
}


/*
 * Sprawdza czy na liscie sprawdzianow jest juz dany przedmiot
 */
int mw_tests_clist_check_test (gchar *name)
{
	GtkWidget *mw_tests_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tests_clist");
	gchar     *test;
	
	int i = 0;
	while (gtk_clist_get_text (GTK_CLIST (mw_tests_clist), i, 0, &test) == 1) {
		if (strcmp (name, test) == 0)
			return 1;
		
		i++;
	}
	
	return 0;
}


/*
 * Usuwa z listy mw_tests_clist, sprawdzian o nazwie "name"
 */
int mw_tests_clist_remove_test (gchar *name)
{
	GtkWidget *mw_tests_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_tests_clist");
	gchar     *test;
	
	int i = 0;
	while (gtk_clist_get_text (GTK_CLIST (mw_tests_clist), i, 0, &test) == 1) {
		if (strcmp (name, test) == 0) {
			gtk_clist_remove (GTK_CLIST (mw_tests_clist), i);
			return 1;
		}
		
		i++;
	}
	
	return 0;
}


/*
 * Funkcja dodaje nowa pozycje do listy kartkowek
 * parametry:
 *            - gubject - nazwa_przedmiotu
 */
int mw_quizes_clist_add (gchar *subject)
{
	if ((strlen (subject) == 0) || (mw_quizes_clist_check_quiz (subject) > 0))
		return 1;
	
	GtkWidget *mw_quizes_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_quizes_clist");
	
	gchar *mw_tmp [] = {subject};
	
	gtk_clist_insert (GTK_CLIST (mw_quizes_clist), 0, mw_tmp);
	
	return 0;
}


/*
 * Funkcja czysci zawartosc listy kartkowek
 */
int mw_quizes_clist_clear ()
{
	GtkWidget *mw_quizes_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_quizes_clist");
	gtk_clist_clear (GTK_CLIST (mw_quizes_clist));
}


/*
 * Funkcja sprawdza czy kartkowka o nazwa zawartej w zmiennej "name" znajduje
 * sie na liscie kartkowek (mw_quizes_clist)
 */
int mw_quizes_clist_check_quiz (gchar *name)
{
	GtkWidget *mw_quizes_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_quizes_clist");
	gchar     *test;
	
	int i = 0;
	while (gtk_clist_get_text (GTK_CLIST (mw_quizes_clist), i, 0, &test) == 1) {
		if (strcmp (name, test) == 0)
			return 1;
		
		i++;
	}
	
	return 0;
}


/*
 * Usuwa z listy mw_quizes_clist, kartkowke o nazwie "name"
 */
int mw_tests_clist_remove_quiz (gchar *name)
{
	GtkWidget *mw_quizes_clist = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_quizes_clist");
	gchar     *test;
	
	int i = 0;
	while (gtk_clist_get_text (GTK_CLIST (mw_quizes_clist), i, 0, &test) == 1) {
		if (strcmp (name, test) == 0) {
			gtk_clist_remove (GTK_CLIST (mw_quizes_clist), i);
			return 1;
		}
		
		i++;
	}
	
	return 0;
}


/*
 * Funkcja pobiera i zwraca zawartosc pola notatek
 */
gchar *mw_notes_textview_get_text ()
{
	GtkWidget     *mw_notes_textview = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_notes_textview");
	GtkTextIter    start, end;
	GtkTextBuffer *text_buffer    = gtk_text_view_get_buffer (GTK_TEXT_VIEW (mw_notes_textview));
	
	gtk_text_buffer_get_bounds (text_buffer, &start, &end);
	
	return gtk_text_buffer_get_text (text_buffer, &start, &end, TRUE);
}


/*
 * Funkcja zmienia zawartosc pola notatek
 * parametry:
 *            - text - tekst wstawiany do pola notatek
 */
int mw_notes_textview_set_text (gchar *text)
{
	GtkWidget     *mw_notes_textview = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_notes_textview");
	GtkTextBuffer *text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (mw_notes_textview));
	gtk_text_buffer_set_text (text_buffer, text, strlen (text));
}


/*
 * Funkcja czysci wszystkie pola wpisu (notatki, klasowki, przedmioty i oceny)
 */
int mw_clear_all ()
{
	mw_marks_clist_clear  ();
	mw_tests_clist_clear  ();
	mw_quizes_clist_clear ();
	
	mw_notes_textview_set_text ("");
}


/*
 * Funkcja sprawdza czy dany przedmiot znajduje sie na liscie przedmiotow ignorowanych
 */
gboolean mw_stat_check_subject (gchar *subject)
{
	GSList *list = gtudb_settings_get_values_lst ("no_stat_subjects");
	gchar  *tmp;
	int     i    = 0;
	
	while (i < g_slist_length (list)) {
		if (strcmp (g_slist_nth (list, i)->data, subject) == 0)
			return TRUE;
		
		i++;
	}
	
	return FALSE;
}


/*
 * Funkcja oblicza srednia z podanych w zmiennej 'marks' ocen.
 */
GString *mw_stat_average (GString *marks, gboolean mod, float plus, float minus)
{
	GString *rtn  = g_string_new ("");
	GString *tmp  = g_string_new ("");
	char     mmod = ' ';
	
	float total = 0, count = 0;
	
	int i = 0;
	while (i < strlen (marks->str)) {
		if (isalnum (marks->str [i]) || isalpha (marks->str [i]) || (marks->str [i] == '+') || (marks->str [i] == '-')) {
			if (((marks->str [i] == '+') || (marks->str [i] == '-')) && (mod == TRUE)) {
				mmod = marks->str [i];
			} else
				tmp = g_string_append_c (tmp, marks->str [i]);
		} else {
			if (strlen (tmp->str) > 0) {
				total += atof (tmp->str);
				count++;
				
				if (mmod == '+')
					total += plus;
				
				if (mmod == '-')
					total += minus;
				
				tmp  = g_string_assign (tmp, "");
				mmod = ' ';
			}
		}
		
		i++;
	}
	
	g_string_append_printf (rtn, "%.2f", (total / count));
	
	return rtn;
}


/*
 * Funkcja oczyszcza oceny z "niepotrzenych informacji"
 */
gchar *mw_stat_clean_marks (gchar *marks, gboolean mod)
{
	if (strlen (marks) < 1)
		return marks;
	
	
	GString *rtn  = g_string_new ("");
	GString *tmp  = g_string_new ("");
	
	int i = 0;
	
	if (mod == TRUE) {
		while (i < strlen (marks)) {
			if (isalnum (marks [i]) || isalpha (marks [i]) || (marks [i] == '+') || (marks [i] == '-')) {
				tmp = g_string_append_c (tmp, marks [i]);
			} else {
				if (strlen (tmp->str) > 0) {
					g_string_append_printf (rtn, "%s, ", tmp->str);
					tmp = g_string_assign (tmp, "");
				}
			}
			
			i++;
		}
		
		if (strlen (tmp->str) > 0)
			g_string_append_printf (rtn, "%s, ", tmp->str);
	} else {
		while (i < strlen (marks)) {
			if (isalnum (marks [i]) || isalpha (marks [i])) {
				tmp = g_string_append_c (tmp, marks [i]);
			} else {
				if (strlen (tmp->str) > 0) {
					g_string_append_printf (rtn, "%s, ", tmp->str);
					tmp = g_string_assign (tmp, "");
				}
			}
			
			i++;
		}
		
		if (strlen (tmp->str) > 0)
			g_string_append_printf (rtn, "%s, ", tmp->str);
	}
	
	
	return rtn->str;
}


/*
 * Funkcja dodaje informacje do listy
 */
GSList *mw_stat_add (GSList *list, gchar *subject, gchar *marks, gboolean mod)
{
	if (strlen (marks) < 1)
		return list;
	
	
	GSList *list2 = list;
	
	struct_stat_data *tmp = g_new (struct_stat_data, 1);
	struct_stat_data *tmp2;
	
	int i = 0;
	while (i < g_slist_length (list2)) {
		tmp2 = g_new (struct_stat_data, 1);
		tmp2 = g_slist_nth (list2, i)->data;
		
		if (strcmp (tmp2->subject->str, subject) == 0) {
			tmp->subject = g_string_new (subject);
			tmp->marks   = g_string_new (g_strdup (tmp2->marks->str));
			tmp->marks   = g_string_append (tmp->marks, mw_stat_clean_marks (marks, mod));
			
			list2 = g_slist_remove_link (list2, g_slist_nth (list, i));
			list2 = g_slist_append (list2, tmp);
			
			return list2;
		}
		
		i++;
	}
	
	
	tmp->subject = g_string_new (subject);
	tmp->marks   = g_string_new (mw_stat_clean_marks (marks, mod));
	list2 = g_slist_append (list2, tmp);
	
	return list2;
}


/*
 * Funkcja tworzy statystyki
 */
int mw_stats_make ()
{
	// Zakrycie interfejsu na czas tworzenia statystyk
	gtk_widget_set_sensitive (GTK_WIDGET (mw_widget), FALSE);
	while (gtk_events_pending ()) gtk_main_iteration ();
	
	// Pobranie informacji z widgetow z ustawieniami statystyk
	GtkWidget *mw_stats_begin_gde  = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_stats_begin_gde");
	GtkWidget *mw_stats_end_gde    = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_stats_end_gde");
	GtkWidget *mw_stats_mod_chkbtn = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_stats_mod_chkbtn");
	GtkWidget *mw_stats_marks_list = (gpointer) lookup_widget (GTK_WIDGET (mw_widget), "mw_stats_marks_list");
	
	GDate *begin = type_date_to_gdate (timet_to_typedate (gnome_date_edit_get_time (GNOME_DATE_EDIT (mw_stats_begin_gde))));
	GDate *end   = type_date_to_gdate (timet_to_typedate (gnome_date_edit_get_time (GNOME_DATE_EDIT (mw_stats_end_gde))));
	
	float    plus  =  atof (gtudb_settings_get_value ("stat_mod_plus"));
	float    minus =  atof (gtudb_settings_get_value ("stat_mod_minus"));
	gboolean mod   = TRUE;
	
	
	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (mw_stats_mod_chkbtn)) == FALSE) {
		plus  = 0;
		minus = 0;
		mod   = FALSE;
	}
	
	
	// Zbieranie informacji
	GSList       *list = NULL;
	GTUDBEntry   *tmp_entry   = g_new (GTUDBEntry, 1);
	GTUDBSubject *tmp_subject = g_new (GTUDBSubject, 1);
	
	int i = 0, j;
	while (i < g_slist_length (gtudb_list)) {
		tmp_entry = g_slist_nth (gtudb_list, i)->data;
		
		if ((g_date_compare (type_date_to_gdate (tmp_entry->date), end) <= 0) &&
			(g_date_compare (type_date_to_gdate (tmp_entry->date), begin) >= 0)) {
			
			j = 0;
			while (j < g_slist_length (tmp_entry->subjects)) {
				tmp_subject = g_slist_nth (tmp_entry->subjects, j)->data;
				
				if (mw_stat_check_subject (tmp_subject->name) == FALSE)
					list = mw_stat_add (list, g_strdup (tmp_subject->name), g_strdup (tmp_subject->marks), mod);
				
				j++;
			}
		}
		
		i++;
	}
	
	
	// wypisywanie informacji
	gtk_clist_clear (GTK_CLIST (mw_stats_marks_list));
	
	
	GdkColor color  = {0, 0xEEEE, 0xEEEE, 0xEEEE};
	GdkColor color2 = {0, 0xFFFF, 0xFFFF, 0xFFFF};
	gdk_color_alloc (gdk_colormap_get_system (), &color);
	gdk_color_alloc (gdk_colormap_get_system (), &color2);
	
	float total = 0;
	int   count = 0;
	i = 0;
	while (i < g_slist_length (list)) {
		struct_stat_data *stat = g_slist_nth (list, i)->data;
		
		if (mw_stat_check_subject (stat->subject->str) == FALSE) {
			gtk_clist_append (GTK_CLIST (mw_stats_marks_list), (gchar * []) {stat->subject->str, stat->marks->str, mw_stat_average (stat->marks, mod, plus, minus)->str});
			total += atof (mw_stat_average (stat->marks, mod, plus, minus)->str);
			count++;
			
			if (count % 2 == 1)
				gtk_clist_set_background (GTK_CLIST (mw_stats_marks_list), i, &color);
			else
				gtk_clist_set_background (GTK_CLIST (mw_stats_marks_list), i, &color2);
		}
		
		i++;
	}
	
	
	if (count > 0) {
		GString *tmp = g_string_new ("");
		g_string_append_printf (tmp, "%.2f", (total / count));
		
		gtk_clist_append (GTK_CLIST (mw_stats_marks_list), (gchar * []) {"", _("Average: "), tmp->str});
	}
	
	
	g_slist_free (list);
	
	
	// Przywrocenie interfejsu
	gtk_widget_set_sensitive (GTK_WIDGET (mw_widget), TRUE);
}


/*
 * Funkcja wyswietla okno ustawien statystyk
 */
int mwss_show ()
{
	mwss_widget = (gpointer) create_main_window_stats_settings ();

	// wypelniamy liste ignorowanych przedmiotow
	GtkWidget *mwss_subjects = (gpointer) lookup_widget (GTK_WIDGET (mwss_widget), "mwss_subjects");
	
	GSList *tmp_list = gtudb_settings_get_values_lst ("no_stat_subjects");
	int     i        = 0;
	
	while (i < g_slist_length (tmp_list)) {
		gtk_clist_append (GTK_CLIST (mwss_subjects), (gchar * []) {g_slist_nth (tmp_list, i)->data});
		i++;
	}
	
	
	// ustawiamy modyfikatory
	GtkWidget *mwss_plus_value  = (gpointer) lookup_widget (GTK_WIDGET (mwss_widget), "mwss_plus_value");
	GtkWidget *mwss_minus_value = (gpointer) lookup_widget (GTK_WIDGET (mwss_widget), "mwss_minus_value");
	
	gtk_entry_set_text (GTK_ENTRY (mwss_plus_value), gtudb_settings_get_value ("stat_mod_plus"));
	gtk_entry_set_text (GTK_ENTRY (mwss_minus_value), gtudb_settings_get_value ("stat_mod_minus"));
	
	
	// wyswietlamy okno
	gtk_widget_show (GTK_WIDGET (mwss_widget));
}


/*
 * Funkcja dodaje przedmiot o nazwie 'name'
 */
int mwss_add_subject (gchar *name)
{
	if (mwss_list_check_subject (name) >= 0)
		return -1;
	
	GtkWidget *mwss_subjects = (gpointer) lookup_widget (GTK_WIDGET (mwss_widget), "mwss_subjects");
	gtk_clist_append (GTK_CLIST (mwss_subjects), (gchar * []) {name});
	gtk_clist_sort (GTK_CLIST (mwss_subjects));
	
	return 0;
}


/*
 * Funkcja usuwa przedmiot o id (na liscie) 'id'
 */
int mwss_remove_subject (gint id)
{
	GtkWidget *mwss_subjects = (gpointer) lookup_widget (GTK_WIDGET (mwss_widget), "mwss_subjects");
	gtk_clist_remove (GTK_CLIST (mwss_subjects), id);
	
	return 0;
}


/*
 * Funkcja zapisuje wszystkie ustawienia z okna main_window_stats_settings
 */
int mwss_ok_btn ()
{
	// tworzymy liste ignorowanych przedmiotow
	GtkWidget *mwss_subjects = (gpointer) lookup_widget (GTK_WIDGET (mwss_widget), "mwss_subjects");
	
	int     i = 0;
	gchar  *tmp;
	GSList *tmp_list = NULL;
	
	while (gtk_clist_get_text (GTK_CLIST (mwss_subjects), i, 0, &tmp) == 1) {
		tmp_list = g_slist_append (tmp_list, g_strdup (tmp));
		i++;
	}
	
	gtudb_settings_set_values_lst ("no_stat_subjects", tmp_list);
	
	// ustawiamy modyfikatory dla ocen
	GtkWidget *mwss_plus_value  = (gpointer) lookup_widget (GTK_WIDGET (mwss_widget), "mwss_plus_value");
	GtkWidget *mwss_minus_value = (gpointer) lookup_widget (GTK_WIDGET (mwss_widget), "mwss_minus_value");
	
	gtudb_settings_set_value ("stat_mod_plus", (gchar *) gtk_entry_get_text (GTK_ENTRY (mwss_plus_value)));
	gtudb_settings_set_value ("stat_mod_minus", (gchar *) gtk_entry_get_text (GTK_ENTRY (mwss_minus_value)));
}


/*
 * Funkcja sprawdza czy na liscie przedmiotow (widgecie) znajudje sie przedmiot
 * o nazwie 'name'
 */
int mwss_list_check_subject (gchar *name)
{
	if (strlen (name) == 0)
		return -1;
	
	
	GtkWidget *mwss_subjects = (gpointer) lookup_widget (GTK_WIDGET (mwss_widget), "mwss_subjects");
	
	int    i = 0;
	gchar *tmp;
	
	while (gtk_clist_get_text (GTK_CLIST (mwss_subjects), i, 0, &tmp) == 1) {
		if (strcmp (tmp, name) == 0)
			return i;
		
		i++;
	}
	
	return -1;
}


/*
 * Funkcja sprawdza czy na liscie (GSList) przedmiotow znajduje sie przedmiot o
 * nazwie 'name'
 */
int mwss_check_subject (gchar *name)
{
	GSList *tmp_list = gtudb_settings_get_values_lst ("no_stat_subjects");
	int     i        = 0;
	
	while (i < g_slist_length (tmp_list)) {
		if (strcmp (name, (gchar *) g_slist_nth (tmp_list, i)->data) == 0)
			return i;
		
		i++;
	}
	
	return -1;
}


/*
 * Funkcja pokazuje okno main_window_tests
 */
int mwt_show ()
{
	gtk_widget_show (GTK_WIDGET (mwt_widget));
	
	GtkWidget *mwt_from_date_edit = (gpointer) lookup_widget (GTK_WIDGET (mwt_widget), "mwt_from_date_edit");
	GtkWidget *mwt_to_date_edit   = (gpointer) lookup_widget (GTK_WIDGET (mwt_widget), "mwt_to_date_edit");
	
	GDate *date_end = type_date_to_gdate (entry_date);
	g_date_add_days (date_end, 30);
	
	gnome_date_edit_set_time (GNOME_DATE_EDIT (mwt_from_date_edit), typedate_to_timet (entry_date));
	gnome_date_edit_set_time (GNOME_DATE_EDIT (mwt_to_date_edit), typedate_to_timet (gdate_to_type_date(date_end)));
}


/*
 * Funkcja wyszukuje sprawdziany i wyswietla wyniki
 */
int mwt_search ()
{
	GtkWidget *mwt_from_date_edit = (gpointer) lookup_widget (GTK_WIDGET (mwt_widget), "mwt_from_date_edit");
	GtkWidget *mwt_to_date_edit   = (gpointer) lookup_widget (GTK_WIDGET (mwt_widget), "mwt_to_date_edit");
	GtkWidget *mwt_tests_label    = (gpointer) lookup_widget (GTK_WIDGET (mwt_widget), "mwt_tests_label");
	
	gtk_label_set_label (GTK_LABEL (mwt_tests_label), "");
	
	time_t date_begin = gnome_date_edit_get_time (GNOME_DATE_EDIT (mwt_from_date_edit));
	time_t date_end   = gnome_date_edit_get_time (GNOME_DATE_EDIT (mwt_to_date_edit));
	
	GSList *tmp = gtudb_search_test (timet_to_typedate (date_begin), timet_to_typedate (date_end));
	int     i   = 0;
	int     j   = 0;
	gchar   str[2048];
	
	sprintf (str, "");
	
	while (i < g_slist_length (tmp)) {
		g_sprintf (str, "%s<b>%s</b>\n", str, g_strdup (g_slist_nth (tmp, i)->data));
		
		GSList *tmp2 = gtudb_entry_get_subjects_list (gdate_to_type_date (text_to_gdate (g_strdup (g_slist_nth (tmp, i)->data))), 2);		

		j = 0;
		while (j < g_slist_length (tmp2)) {
			g_sprintf (str, _("%s\t%s (class-test)\n"), str, g_strdup (g_slist_nth (tmp2, j)->data));
			j++;
		}
		
		tmp2 = gtudb_entry_get_subjects_list (gdate_to_type_date (text_to_gdate (g_strdup (g_slist_nth (tmp, i)->data))), 1);
		
		j = 0;
		while (j < g_slist_length (tmp2)) {
			sprintf (str, _("%s\t%s (quiz)\n"), str, g_strdup (g_slist_nth (tmp2, j)->data));
			j++;
		}
		
		sprintf (str, "%s\n", str);
		
		i++;
	}
	
	if (strlen (str) == 0)
		sprintf (str, _("<b>No results found !</b>\n"));
	
	gtk_label_set_label (GTK_LABEL (mwt_tests_label), str);
}
